/*****************************************************************************
 *
 *  jobmanager.cpp -- JobManager is an asynchronous job handler.
 *
 *  Copyright (C) 2010 Starbuck <starbuck@labdv.com>
 *  All rights reserved.
 *
 *  This file is part of the JobManager module of the DVDx 4 application.
 *
 *  GNU Lesser General Public License Usage
 *  This file may be used under the terms of the GNU Lesser General
 *  Public License version 2.1 as published by the Free Software
 *  Foundation and appearing in the file LICENSE.LGPL included in the
 *  packaging of this file.  Please review the following information to
 *  ensure the GNU Lesser General Public License version 2.1 requirements
 *  will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 *
 *  GNU General Public License Usage
 *  Alternatively, this file may be used under the terms of the GNU
 *  General Public License version 3.0 as published by the Free Software
 *  Foundation and appearing in the file LICENSE.GPL included in the
 *  packaging of this file.  Please review the following information to
 *  ensure the GNU General Public License version 3.0 requirements will be
 *  met: http://www.gnu.org/copyleft/gpl.html.
 *
 *****************************************************************************/

#include "jobmanager.h"
#include "macros.h"
#include "../main/application.h"
#include <QProcess>
#include <QSignalMapper>
#include <QFileInfo>
#include <QDebug>
#include <QCoreApplication>
#include <QLibrary>

JobManager::JobManager(QTextEdit *stdIn, QTextEdit *stdOut, QTextEdit *stdErr, QTextEdit *defLog, 
	const int verbosityLevel, const int latencyStart, const int latencyFinished, const int latencyKill)
{
	LatencyStart = (latencyStart == 0) ? DefaultLatencyStart : latencyStart;
	LatencyFinished = (latencyFinished == 0) ? DefaultLatencyFinished : latencyFinished;
	LatencyKill = (latencyKill == 0) ? DefaultLatencyKill : latencyKill;

	// Set IO channels from parent channels
	standardInput = stdIn;
	standardOutput = stdOut;
	standardError = stdErr;
	log = defLog;
	verbosity = verbosityLevel;
}

void JobManager::setLatency(const int latencyStart, const int latencyFinished, const int latencyKill)
{
	LatencyStart = (latencyStart == 0) ? DefaultLatencyStart : latencyStart;
	LatencyFinished = (latencyFinished == 0) ? DefaultLatencyFinished : latencyFinished;
	LatencyKill = (latencyKill == 0) ? DefaultLatencyKill : latencyKill;
}

void JobManager::setVerbosity(const int verbose)
{
	verbosity = verbose;
}

void JobManager::addJob( Job *job )
{
	// Append job
	jobs.append( job );

	processJobs();
}

void JobManager::setProcessDone(Process *process)
{
	Task *t = process->getTask();
	if ( t->getState() == Task::Terminated ) t->setState( Task::Done );
}

void JobManager::processJobs()
{
	qDebug("JobManager: processing.");

	if ( jobs.isEmpty() ) return;

	QListIterator<Job *> j(jobs);

	while (j.hasNext())
	{
		Job *job = j.next();

		Task *taskToStart = job->getTaskToStart();

		if (taskToStart == 0) continue;

		taskToStart->setState( Task::Submitting );  // avoid being submitted elsewhere (multi-instances, multi-threading paranoia)

		QString cmd = taskToStart->getCommand();
		QStringList args = taskToStart->getArguments();
		QString workingDir = taskToStart->getDirectory();
		QString taskName = taskToStart->getName();
		Job *ref = (Job *)taskToStart->getJobRef();  // might be job here (from j.next())
		QString jobId = taskToStart->getJobId();
		QString jobName = taskToStart->getJobName();
		
		QFileInfo fi (cmd);
		QString cmdBaseName = fi.completeBaseName() + "." + fi.suffix();

		// Start task!
		QString strIn(cmd);
		strIn.append(" ").append(args.join(" "));
		printIn(strIn);

		qDebug() << "JobManager:"<< jobId << "start" << cmd << args;

		Process *process = new Process();

#ifdef Q_OS_MAC

#elif defined(Q_OS_LINUX)
		// Will is work with MEncoder distro?
		QProcessEnvironment env = QProcessEnvironment::systemEnvironment();
		env.insert("LD_LIBRARY_PATH", "/opt/dvdx4/lib");
		process->setProcessEnvironment(env);
#endif

		process->setWorkingDirectory(workingDir);
		process->start(cmd, args);

		if (!process->waitForStarted( LatencyStart ))
		{
			taskToStart->setState( Task::StartFailure );  // failed to start
			process->close();
			delete(process);

			qDebug() << "JobManager:" << jobId << "failed to start" << cmd << args;
		
			printLog( QObject::tr("Failed to start %1 [%2].")
				.arg(taskName).arg(jobId), VERBOSITY_QUIET );
		}
		else
		{
			process->setJob( job );
			process->setTask( taskToStart );
			process->setCommand( &cmdBaseName );
			
			processes.append(process);

			Q_PID pid = process->pid();
			taskToStart->setPid( pid );  // task is now running with that PID
			taskToStart->setProcess( (void *)process );

			taskToStart->setState( Task::Running );

			connect ( taskToStart, SIGNAL(killProcess()), process, SLOT(kill()) );
			//*experimental connect ( taskToStart, SIGNAL(stopProcess()), process, SLOT(interrupt()) );

			connect(process, SIGNAL(finished(int, QProcess::ExitStatus)), 
				this, SLOT(handleProcessFinished(int, QProcess::ExitStatus)));

			connect(process, SIGNAL(readyReadStandardOutput()), 
				this, SLOT(readProcessStdOutput()));
			connect(process, SIGNAL(readyReadStandardError()), 
				this, SLOT(readProcessStdError()));

			printLog( QObject::tr("Started job %1 [%2].")
				.arg(taskName).arg(jobId), VERBOSITY_QUIET );

			Application::setKeepAwake();

			emit processStarted( taskToStart );
		}
	}
}

void JobManager::abortProcesses()
{
	int n = processes.count();

	for (int i = 0; i < n; i++)
	{
		processes.takeFirst()->close();
	}

	processes.clear();

	QCoreApplication::processEvents();  // we want to process all to ensure deleting output files
}

bool JobManager::isIdle()
{
	bool idle;
	
	if ( jobs.isEmpty() )
	{
		idle = true;
	}
	else
	{
		idle = true;
		QListIterator<Job *> j(jobs);
		while ( j.hasNext() )
		{
			idle &= j.next()->isFinished();
		}
	}

	return idle;
}

void JobManager::handleProcessFinished(int exitCode, QProcess::ExitStatus exitStatus)
{
	Process *process = (Process*)sender();

	qDebug() << "JobManager: exit code" << process->getJob()->getName() << process->exitCode();

	bool terminated = (exitStatus == QProcess::NormalExit) && (exitCode == 0);
	process->getJob()->updateOnTerminatedTask( process->getTask(), terminated );

	Application::restoreFromKeepAwake();

	emit processFinished( process, exitCode, exitStatus );
}

void JobManager::readProcessStdOutput()
{
	emit processStdOutReady( (Process*)sender() );
}

void JobManager::readProcessStdError()
{
	emit processStdErrReady( (Process*)sender() );
}

void JobManager::printLog(const QString &message, const int verbosityLevel)
{
	if ( log == 0 )
		qDebug() << "JobManager Log:" << message;
	else if ( verbosityLevel <= verbosity )
	{
		log->append( message );
		QCoreApplication::processEvents();  // we want to show message immediately
	}
}

void JobManager::printIn(const QString &message, const int verbosityLevel)
{
	if ( standardInput == 0)
		qDebug() << "JobManager stdIn:" << message;
	else if ( verbosityLevel <= verbosity )
	{
		standardInput->append( message );
		QCoreApplication::processEvents();  // we want to show message immediately
	}
}

void JobManager::printOut(const QString &message, const int verbosityLevel)
{
	if ( standardOutput == 0)
		qDebug() << "JobManager stdOut:" << message;
	else if ( verbosityLevel <= verbosity )
	{
		standardOutput->append( message );
		QCoreApplication::processEvents();  // we want to show message immediately
	}
}

void JobManager::printErr(const QString &message, const int verbosityLevel)
{
	if ( standardError == 0)
		qDebug() << "JobManager stdErr:" << message;
	else if ( verbosityLevel <= verbosity )
		standardError->append( message );
}

void JobManager::clearLog()
{
	if ( log != 0)
		log->clear();
}

void JobManager::clearIn()
{
	if ( standardInput != 0)
		standardInput->clear();
}

void JobManager::clearOut()
{
	if ( standardOutput != 0)
		standardOutput->clear();
}

void JobManager::clearErr()
{
	if ( standardError != 0)
		standardError->clear();
}
